using Microsoft.Extensions.Configuration;
using System.Text.Json;

namespace View_by_Distance.Rename.Models.Binder;

public class AppSettings
{

#nullable disable

    public string Company { get; set; }
    public string ComparePathsFile { get; set; }
    public string CopyTo { get; set; }
    public int? MaxDegreeOfParallelism { get; set; }
    public int? MaxMinutesDelta { get; set; }
    public bool? RenameUndo { get; set; }
    public string WorkingDirectoryName { get; set; }

#nullable restore

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    private static Models.AppSettings Get(AppSettings? appSettings)
    {
        Models.AppSettings result;
        if (appSettings?.MaxDegreeOfParallelism is null)
            throw new NullReferenceException(nameof(appSettings.MaxDegreeOfParallelism));
        if (appSettings?.MaxMinutesDelta is null)
            throw new NullReferenceException(nameof(appSettings.MaxMinutesDelta));
        if (appSettings?.RenameUndo is null)
            throw new NullReferenceException(nameof(appSettings.RenameUndo));
        result = new(
            appSettings.Company,
            appSettings.ComparePathsFile,
            appSettings.CopyTo,
            appSettings.MaxDegreeOfParallelism.Value,
            appSettings.MaxMinutesDelta.Value,
            appSettings.RenameUndo.Value,
            appSettings.WorkingDirectoryName
        );
        return result;
    }

    public static Models.AppSettings Get(IConfigurationRoot configurationRoot)
    {
        Models.AppSettings result;
        AppSettings? appSettings = configurationRoot.Get<AppSettings>();
        result = Get(appSettings);
        return result;
    }

}