using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Shared.Models;

public class PersonContainer : Properties.IPersonContainer
{

    public int? ApproximateYears { init; get; }
    public PersonBirthday[]? Birthdays { init; get; }
    public string[] DisplayDirectoryAllFiles { init; get; }
    public string DisplayDirectoryName { init; get; }
    public long? Key { init; get; }
    public bool? KeyIsMaxBirthday { init; get; }
    public Person? Person { init; get; }
    public PersonDirectory? PersonDirectory { init; get; }

    [JsonConstructor]
    public PersonContainer(int? approximateYears, PersonBirthday[]? birthdays, string[] displayDirectoryAllFiles, string displayDirectoryName, long? key, Person? person, PersonDirectory? personDirectory)
    {
        ApproximateYears = approximateYears;
        Birthdays = birthdays;
        DisplayDirectoryAllFiles = displayDirectoryAllFiles;
        DisplayDirectoryName = displayDirectoryName;
        Key = key;
        Person = person;
        PersonDirectory = personDirectory;
        KeyIsMaxBirthday = birthdays is null || key is null ? null : key.Value == birthdays.First().Value.Ticks;
    }

    public PersonContainer(string mappingDefaultName, char[] personCharacters, PersonBirthday birthday, string displayDirectoryName) :
        this(Stateless.Methods.IAge.GetApproximateYears(personCharacters, displayDirectoryName), new PersonBirthday[] { birthday }, Array.Empty<string>(), displayDirectoryName, birthday.Value.Ticks, Stateless.Methods.IPerson.GetPerson(mappingDefaultName, personCharacters, displayDirectoryName, birthday.Value.Ticks, birthday), null)
    { }

    public PersonContainer(int? approximateYears, PersonBirthday birthdays, string displayDirectoryName, long key) :
        this(approximateYears, new PersonBirthday[] { birthdays }, Array.Empty<string>(), displayDirectoryName, key, null, null)
    { }

    public PersonContainer(int? approximateYears, PersonBirthday birthdays, PersonDirectory? personDirectory, string displayDirectoryName, long key) :
        this(approximateYears, new PersonBirthday[] { birthdays }, Array.Empty<string>(), displayDirectoryName, key, null, personDirectory)
    { }

    public PersonContainer(int? approximateYears, string[] displayDirectoryAllFiles, string displayDirectoryName, PersonDirectory? personDirectory) :
        this(approximateYears, null, displayDirectoryAllFiles, displayDirectoryName, null, null, personDirectory)
    { }

    public PersonContainer(int? approximateYears, PersonBirthday[]? birthdays, string[] displayDirectoryAllFiles, string displayDirectoryName, long? key, Person? person) :
        this(approximateYears, birthdays, displayDirectoryAllFiles, displayDirectoryName, key, person, null)
    { }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

}