using System.Text;

namespace View_by_Distance.Shared.Models.Stateless.Methods;

internal abstract class PersonName
{

    // ...

    internal static Models.PersonName GetPersonName(string name)
    {
        Models.PersonName result;
        Models.PersonNameLast personNameLast;
        Models.PersonNameAlias personNameAlias;
        Models.PersonNameFirst personNameFirst;
        Models.PersonNameMiddle personNameMiddle;
        string[] segments = name.Split(' ');
        if (segments.Length == 1)
        {
            personNameFirst = new(string.Empty);
            personNameLast = new(string.Empty);
            personNameMiddle = new(string.Empty);
            personNameAlias = new(segments[0]);
        }
        else if (segments.Length == 2)
        {
            personNameFirst = new(segments[0]);
            personNameLast = new(segments[1]);
            personNameMiddle = new(string.Empty);
            personNameAlias = new(string.Empty);
        }
        else if (segments.Length == 3 && segments[2] is "Jr." or "Jr" or "Sr")
        {
            personNameFirst = new(segments[0]);
            personNameLast = new(segments[1]);
            personNameMiddle = new(string.Empty);
            personNameAlias = new(string.Join(' ', segments));
        }
        else
        {
            string[] comment = name.Split(new string[] { " (" }, StringSplitOptions.None);
            if (comment.Length == 1)
            {
                personNameFirst = new(segments[0]);
                personNameLast = new(segments[^1]);
                personNameMiddle = new(string.Empty);
                personNameAlias = new(string.Join(' ', segments));
            }
            else
            {
                segments = comment[0].Split(' ');
                personNameFirst = new(segments[0]);
                personNameLast = new(segments[^1]);
                personNameMiddle = new(string.Empty);
                personNameAlias = new(string.Concat(string.Join(' ', segments), " (", comment[1]));
            }
        }
        result = new(personNameFirst, personNameMiddle, personNameLast, personNameAlias);
        return result;
    }

    internal static string GetFullName(Models.PersonName personName)
    {
        StringBuilder result = new();
        if (personName.First is not null)
            _ = result.Append(personName.First.Value);
        if (personName.Middle is not null)
            _ = result.Append(' ').Append(personName.Middle.Value);
        if (personName.Last is not null)
            _ = result.Append(' ').Append(personName.Last.Value);
        return result.ToString();
    }

    internal static Models.PersonName? GetPersonName(Models.GenealogicalDataCommunication genealogicalDataCommunication)
    {
        Models.PersonName? result;
        string[] surNameSegments = genealogicalDataCommunication.SurName is null ? Array.Empty<string>() : genealogicalDataCommunication.SurName.Split(' ');
        string[] givenNameSegments = genealogicalDataCommunication.GivenName is null ? Array.Empty<string>() : genealogicalDataCommunication.GivenName.Split(' ');
        if (!givenNameSegments.Any() || !surNameSegments.Any())
            result = null;
        else
            result = new(new(givenNameSegments[0]), new(givenNameSegments.Length == 1 ? string.Empty : givenNameSegments[1]), new(surNameSegments[^1]), new(string.Empty));
        return result;
    }

}